@@ -1 +1,5 @@
 from agno.models.fireworks.fireworks import Fireworks
+
+__all__ = [
+    "Fireworks",
+]
