@@ -1,18 +1,21 @@
 import json
 import time
+from collections.abc import AsyncIterator
 from dataclasses import dataclass
 from os import getenv
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Type, Union
+from typing import Any, Dict, Iterator, List, Optional, Type, Union
 from uuid import uuid4
 
 from pydantic import BaseModel
 
 from agno.exceptions import ModelProviderError
-from agno.media import Audio, File, ImageArtifact, Video
+from agno.media import Audio, File, Image, Video
 from agno.models.base import Model
-from agno.models.message import Citations, Message, MessageMetrics, UrlCitation
+from agno.models.message import Citations, Message, UrlCitation
+from agno.models.metrics import Metrics
 from agno.models.response import ModelResponse
+from agno.run.agent import RunOutput
 from agno.utils.gemini import convert_schema, format_function_definitions, format_image_for_message
 from agno.utils.log import log_debug, log_error, log_info, log_warning
 from agno.utils.models.schema_utils import get_response_schema_for_provider
@@ -30,8 +33,11 @@ try:
         GoogleSearch,
         GoogleSearchRetrieval,
         Part,
+        Retrieval,
         ThinkingConfig,
         Tool,
+        UrlContext,
+        VertexAISearch,
     )
     from google.genai.types import (
         File as GeminiFile,
@@ -68,6 +74,9 @@ class Gemini(Model):
     search: bool = False
     grounding: bool = False
     grounding_dynamic_threshold: Optional[float] = None
+    url_context: bool = False
+    vertexai_search: bool = False
+    vertexai_search_datastore: Optional[str] = None
 
     temperature: Optional[float] = None
     top_p: Optional[float] = None
@@ -115,7 +124,6 @@ class Gemini(Model):
         """
         if self.client:
             return self.client
-
         client_params: Dict[str, Any] = {}
         vertexai = self.vertexai or getenv("GOOGLE_GENAI_USE_VERTEXAI", "false").lower() == "true"
 
@@ -199,26 +207,45 @@ class Gemini(Model):
         if thinking_config_params:
             config["thinking_config"] = ThinkingConfig(**thinking_config_params)
 
-        if self.grounding and self.search:
-            log_info("Both grounding and search are enabled. Grounding will take precedence.")
-            self.search = False
+        # Build tools array based on enabled built-in tools
+        builtin_tools = []
 
         if self.grounding:
-            log_info("Grounding enabled. External tools will be disabled.")
-            config["tools"] = [
+            log_info(
+                "Grounding enabled. This is a legacy tool. For Gemini 2.0+ Please use enable `search` flag instead."
+            )
+            builtin_tools.append(
                 Tool(
                     google_search=GoogleSearchRetrieval(
                         dynamic_retrieval_config=DynamicRetrievalConfig(
                             dynamic_threshold=self.grounding_dynamic_threshold
                         )
                     )
-                ),
-            ]
+                )
+            )
 
-        elif self.search:
-            log_info("Search enabled. External tools will be disabled.")
-            config["tools"] = [Tool(google_search=GoogleSearch())]
+        if self.search:
+            log_info("Google Search enabled.")
+            builtin_tools.append(Tool(google_search=GoogleSearch()))
 
+        if self.url_context:
+            log_info("URL context enabled.")
+            builtin_tools.append(Tool(url_context=UrlContext()))
+
+        if self.vertexai_search:
+            log_info("Vertex AI Search enabled.")
+            if not self.vertexai_search_datastore:
+                log_error("vertexai_search_datastore must be provided when vertexai_search is enabled.")
+                raise ValueError("vertexai_search_datastore must be provided when vertexai_search is enabled.")
+            builtin_tools.append(
+                Tool(retrieval=Retrieval(vertex_ai_search=VertexAISearch(datastore=self.vertexai_search_datastore)))
+            )
+
+        # Set tools in config
+        if builtin_tools:
+            if tools:
+                log_info("Built-in tools enabled. External tools will be disabled.")
+            config["tools"] = builtin_tools
         elif tools:
             config["tools"] = [format_function_definitions(tools)]
 
@@ -238,21 +265,33 @@ class Gemini(Model):
     def invoke(
         self,
         messages: List[Message],
+        assistant_message: Message,
         response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
         tools: Optional[List[Dict[str, Any]]] = None,
         tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
-    ):
+        run_response: Optional[RunOutput] = None,
+    ) -> ModelResponse:
         """
         Invokes the model with a list of messages and returns the response.
         """
         formatted_messages, system_message = self._format_messages(messages)
         request_kwargs = self.get_request_params(system_message, response_format=response_format, tools=tools)
         try:
-            return self.get_client().models.generate_content(
+            if run_response and run_response.metrics:
+                run_response.metrics.set_time_to_first_token()
+
+            assistant_message.metrics.start_timer()
+            provider_response = self.get_client().models.generate_content(
                 model=self.id,
                 contents=formatted_messages,
                 **request_kwargs,
             )
+            assistant_message.metrics.stop_timer()
+
+            model_response = self._parse_provider_response(provider_response, response_format=response_format)
+
+            return model_response
+
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             error_message = str(e.response) if hasattr(e, "response") else str(e)
@@ -269,10 +308,12 @@ class Gemini(Model):
     def invoke_stream(
         self,
         messages: List[Message],
+        assistant_message: Message,
         response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
         tools: Optional[List[Dict[str, Any]]] = None,
         tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
-    ):
+        run_response: Optional[RunOutput] = None,
+    ) -> Iterator[ModelResponse]:
         """
         Invokes the model with a list of messages and returns the response as a stream.
         """
@@ -280,11 +321,19 @@ class Gemini(Model):
 
         request_kwargs = self.get_request_params(system_message, response_format=response_format, tools=tools)
         try:
-            yield from self.get_client().models.generate_content_stream(
+            if run_response and run_response.metrics:
+                run_response.metrics.set_time_to_first_token()
+
+            assistant_message.metrics.start_timer()
+            for response in self.get_client().models.generate_content_stream(
                 model=self.id,
                 contents=formatted_messages,
                 **request_kwargs,
-            )
+            ):
+                yield self._parse_provider_response_delta(response)
+
+            assistant_message.metrics.stop_timer()
+
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             raise ModelProviderError(
@@ -300,10 +349,12 @@ class Gemini(Model):
     async def ainvoke(
         self,
         messages: List[Message],
+        assistant_message: Message,
         response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
         tools: Optional[List[Dict[str, Any]]] = None,
         tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
-    ):
+        run_response: Optional[RunOutput] = None,
+    ) -> ModelResponse:
         """
         Invokes the model with a list of messages and returns the response.
         """
@@ -312,11 +363,21 @@ class Gemini(Model):
         request_kwargs = self.get_request_params(system_message, response_format=response_format, tools=tools)
 
         try:
-            return await self.get_client().aio.models.generate_content(
+            if run_response and run_response.metrics:
+                run_response.metrics.set_time_to_first_token()
+
+            assistant_message.metrics.start_timer()
+            provider_response = await self.get_client().aio.models.generate_content(
                 model=self.id,
                 contents=formatted_messages,
                 **request_kwargs,
             )
+            assistant_message.metrics.stop_timer()
+
+            model_response = self._parse_provider_response(provider_response, response_format=response_format)
+
+            return model_response
+
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             raise ModelProviderError(
@@ -332,10 +393,12 @@ class Gemini(Model):
     async def ainvoke_stream(
         self,
         messages: List[Message],
+        assistant_message: Message,
         response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
         tools: Optional[List[Dict[str, Any]]] = None,
         tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
-    ):
+        run_response: Optional[RunOutput] = None,
+    ) -> AsyncIterator[ModelResponse]:
         """
         Invokes the model with a list of messages and returns the response as a stream.
         """
@@ -344,13 +407,21 @@ class Gemini(Model):
         request_kwargs = self.get_request_params(system_message, response_format=response_format, tools=tools)
 
         try:
+            if run_response and run_response.metrics:
+                run_response.metrics.set_time_to_first_token()
+
+            assistant_message.metrics.start_timer()
+
             async_stream = await self.get_client().aio.models.generate_content_stream(
                 model=self.id,
                 contents=formatted_messages,
                 **request_kwargs,
             )
             async for chunk in async_stream:
-                yield chunk
+                yield self._parse_provider_response_delta(chunk)
+
+            assistant_message.metrics.stop_timer()
+
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             raise ModelProviderError(
@@ -388,7 +459,10 @@ class Gemini(Model):
             message_parts: List[Any] = []
 
             # Function calls
-            if (not content or role == "model") and message.tool_calls is not None and len(message.tool_calls) > 0:
+            if role == "model" and message.tool_calls is not None and len(message.tool_calls) > 0:
+                if content is not None:
+                    content_str = content if isinstance(content, str) else str(content)
+                    message_parts.append(Part.from_text(text=content_str))
                 for tool_call in message.tool_calls:
                     message_parts.append(
                         Part.from_function_call(
@@ -396,7 +470,7 @@ class Gemini(Model):
                             args=json.loads(tool_call["function"]["arguments"]),
                         )
                     )
-            # Function results
+            # Function call results
             elif message.tool_calls is not None and len(message.tool_calls) > 0:
                 for tool_call in message.tool_calls:
                     message_parts.append(
@@ -485,9 +559,14 @@ class Gemini(Model):
             return Part.from_bytes(mime_type=mime_type, data=audio.content)
 
         # Case 2: Audio is an url
-        elif audio.url is not None and audio.audio_url_content is not None:
-            mime_type = f"audio/{audio.format}" if audio.format else "audio/mp3"
-            return Part.from_bytes(mime_type=mime_type, data=audio.audio_url_content)
+        elif audio.url is not None:
+            audio_bytes = audio.get_content_bytes()  # type: ignore
+            if audio_bytes is not None:
+                mime_type = f"audio/{audio.format}" if audio.format else "audio/mp3"
+                return Part.from_bytes(mime_type=mime_type, data=audio_bytes)
+            else:
+                log_warning(f"Failed to download audio from {audio}")
+                return None
 
         # Case 3: Audio is a local file path
         elif audio.filepath is not None:
@@ -669,7 +748,7 @@ class Gemini(Model):
         """
         combined_content: List = []
         combined_function_result: List = []
-        message_metrics = MessageMetrics()
+        message_metrics = Metrics()
         if len(function_call_results) > 0:
             for result in function_call_results:
                 combined_content.append(result.content)
@@ -683,7 +762,7 @@ class Gemini(Model):
                 )
             )
 
-    def parse_provider_response(self, response: GenerateContentResponse, **kwargs) -> ModelResponse:
+    def _parse_provider_response(self, response: GenerateContentResponse, **kwargs) -> ModelResponse:
         """
         Parse the OpenAI response into a ModelResponse.
 
@@ -738,8 +817,10 @@ class Gemini(Model):
                                 model_response.content += content_str
 
                 if hasattr(part, "inline_data") and part.inline_data is not None:
-                    model_response.image = ImageArtifact(
-                        id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type
+                    if model_response.images is None:
+                        model_response.images = []
+                    model_response.images.append(
+                        Image(id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type)
                     )
 
                 # Extract function call if present
@@ -758,38 +839,58 @@ class Gemini(Model):
 
                     model_response.tool_calls.append(tool_call)
 
-            if response.candidates and response.candidates[0].grounding_metadata is not None:
-                citations = Citations()
-                grounding_metadata = response.candidates[0].grounding_metadata.model_dump()
-                citations.raw = grounding_metadata
+            citations = Citations()
+            citations_raw = {}
+            citations_urls = []
 
-                # Extract url and title
-                chunks = grounding_metadata.pop("grounding_chunks", None) or []
-                citation_pairs = [
-                    (chunk.get("web", {}).get("uri"), chunk.get("web", {}).get("title"))
-                    for chunk in chunks
-                    if chunk.get("web", {}).get("uri")
-                ]
+            if response.candidates and response.candidates[0].grounding_metadata is not None:
+                grounding_metadata = response.candidates[0].grounding_metadata.model_dump()
+                citations_raw["grounding_metadata"] = grounding_metadata
+
+                chunks = grounding_metadata.get("grounding_chunks", []) or []
+                citation_pairs = []
+                for chunk in chunks:
+                    if not isinstance(chunk, dict):
+                        continue
+                    web = chunk.get("web")
+                    if not isinstance(web, dict):
+                        continue
+                    uri = web.get("uri")
+                    title = web.get("title")
+                    if uri:
+                        citation_pairs.append((uri, title))
 
                 # Create citation objects from filtered pairs
-                citations.urls = [UrlCitation(url=url, title=title) for url, title in citation_pairs]
+                grounding_urls = [UrlCitation(url=url, title=title) for url, title in citation_pairs]
+                citations_urls.extend(grounding_urls)
 
+            # Handle URLs from URL context tool
+            if (
+                response.candidates
+                and hasattr(response.candidates[0], "url_context_metadata")
+                and response.candidates[0].url_context_metadata is not None
+            ):
+                url_context_metadata = response.candidates[0].url_context_metadata.model_dump()
+                citations_raw["url_context_metadata"] = url_context_metadata
+
+                url_metadata_list = url_context_metadata.get("url_metadata", [])
+                for url_meta in url_metadata_list:
+                    retrieved_url = url_meta.get("retrieved_url")
+                    status = url_meta.get("url_retrieval_status", "UNKNOWN")
+                    if retrieved_url and status == "URL_RETRIEVAL_STATUS_SUCCESS":
+                        # Avoid duplicate URLs
+                        existing_urls = [citation.url for citation in citations_urls]
+                        if retrieved_url not in existing_urls:
+                            citations_urls.append(UrlCitation(url=retrieved_url, title=retrieved_url))
+
+            if citations_raw or citations_urls:
+                citations.raw = citations_raw if citations_raw else None
+                citations.urls = citations_urls if citations_urls else None
                 model_response.citations = citations
 
         # Extract usage metadata if present
         if hasattr(response, "usage_metadata") and response.usage_metadata is not None:
-            usage: GenerateContentResponseUsageMetadata = response.usage_metadata
-
-            output_tokens = usage.candidates_token_count or 0
-            if hasattr(usage, "thoughts_token_count") and usage.thoughts_token_count is not None:
-                output_tokens += usage.thoughts_token_count or 0
-
-            model_response.response_usage = {
-                "input_tokens": usage.prompt_token_count or 0,
-                "output_tokens": output_tokens,
-                "total_tokens": usage.total_token_count or 0,
-                "cached_tokens": usage.cached_content_token_count or 0,
-            }
+            model_response.response_usage = self._get_metrics(response.usage_metadata)
 
         # If we have no content but have a role, add a default empty content
         if model_response.role and model_response.content is None and not model_response.tool_calls:
@@ -797,7 +898,7 @@ class Gemini(Model):
 
         return model_response
 
-    def parse_provider_response_delta(self, response_delta: GenerateContentResponse) -> ModelResponse:
+    def _parse_provider_response_delta(self, response_delta: GenerateContentResponse) -> ModelResponse:
         model_response = ModelResponse()
 
         if response_delta.candidates and len(response_delta.candidates) > 0:
@@ -828,8 +929,10 @@ class Gemini(Model):
                                 model_response.content += text_content
 
                     if hasattr(part, "inline_data") and part.inline_data is not None:
-                        model_response.image = ImageArtifact(
-                            id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type
+                        if model_response.images is None:
+                            model_response.images = []
+                        model_response.images.append(
+                            Image(id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type)
                         )
 
                     # Extract function call if present
@@ -855,11 +958,17 @@ class Gemini(Model):
 
                 # Extract url and title
                 chunks = grounding_metadata.pop("grounding_chunks", None) or []
-                citation_pairs = [
-                    (chunk.get("web", {}).get("uri"), chunk.get("web", {}).get("title"))
-                    for chunk in chunks
-                    if chunk.get("web", {}).get("uri")
-                ]
+                citation_pairs = []
+                for chunk in chunks:
+                    if not isinstance(chunk, dict):
+                        continue
+                    web = chunk.get("web")
+                    if not isinstance(web, dict):
+                        continue
+                    uri = web.get("uri")
+                    title = web.get("title")
+                    if uri:
+                        citation_pairs.append((uri, title))
 
                 # Create citation objects from filtered pairs
                 citations.urls = [UrlCitation(url=url, title=title) for url, title in citation_pairs]
@@ -868,18 +977,7 @@ class Gemini(Model):
 
             # Extract usage metadata if present
             if hasattr(response_delta, "usage_metadata") and response_delta.usage_metadata is not None:
-                usage: GenerateContentResponseUsageMetadata = response_delta.usage_metadata
-
-                output_tokens = usage.candidates_token_count or 0
-                if hasattr(usage, "thoughts_token_count") and usage.thoughts_token_count is not None:
-                    output_tokens += usage.thoughts_token_count or 0
-
-                model_response.response_usage = {
-                    "input_tokens": usage.prompt_token_count or 0,
-                    "output_tokens": output_tokens,
-                    "total_tokens": usage.total_token_count or 0,
-                    "cached_tokens": usage.cached_content_token_count or 0,
-                }
+                model_response.response_usage = self._get_metrics(response_delta.usage_metadata)
 
         return model_response
 
@@ -920,3 +1018,28 @@ class Gemini(Model):
         setattr(new_instance, "client", None)
 
         return new_instance
+
+    def _get_metrics(self, response_usage: GenerateContentResponseUsageMetadata) -> Metrics:
+        """
+        Parse the given Google Gemini usage into an Agno Metrics object.
+
+        Args:
+            response_usage: Usage data from Google Gemini
+
+        Returns:
+            Metrics: Parsed metrics data
+        """
+        metrics = Metrics()
+
+        metrics.input_tokens = response_usage.prompt_token_count or 0
+        metrics.output_tokens = response_usage.candidates_token_count or 0
+        metrics.total_tokens = metrics.input_tokens + metrics.output_tokens
+        if response_usage.thoughts_token_count is not None:
+            metrics.output_tokens += response_usage.thoughts_token_count or 0
+
+        metrics.cache_read_tokens = response_usage.cached_content_token_count or 0
+
+        if response_usage.traffic_type is not None:
+            metrics.provider_metrics = {"traffic_type": response_usage.traffic_type}
+
+        return metrics
