@@ -9,7 +9,7 @@ from agno.utils.log import log_debug
 
 
 @dataclass
-class vLLM(OpenAILike):
+class VLLM(OpenAILike):
     """
     Class for interacting with vLLM models via OpenAI-compatible API.
 
@@ -26,8 +26,8 @@ class vLLM(OpenAILike):
     """
 
     id: str = "not-set"
-    name: str = "vLLM"
-    provider: str = "vLLM"
+    name: str = "VLLM"
+    provider: str = "VLLM"
 
     api_key: Optional[str] = getenv("VLLM_API_KEY") or "EMPTY"
     base_url: Optional[str] = getenv("VLLM_BASE_URL", "http://localhost:8000/v1/")
