@@ -10,44 +10,29 @@ from agno.utils.log import log_info, logger
 class LinearTools(Toolkit):
     def __init__(
         self,
-        get_user_details: bool = True,
-        get_teams_details: bool = True,
-        get_issue_details: bool = True,
-        create_issue: bool = True,
-        update_issue: bool = True,
-        get_user_assigned_issues: bool = True,
-        get_workflow_issues: bool = True,
-        get_high_priority_issues: bool = True,
+        api_key: Optional[str] = None,
         **kwargs,
     ):
-        self.api_token = getenv("LINEAR_API_KEY")
+        self.api_key = api_key or getenv("LINEAR_API_KEY")
 
-        if not self.api_token:
-            api_error_message = "API token 'LINEAR_API_KEY' is missing. Please set it as an environment variable."
-            logger.error(api_error_message)
+        if not self.api_key:
+            raise ValueError("Linear API key is required")
 
         self.endpoint = "https://api.linear.app/graphql"
-        self.headers = {"Authorization": f"{self.api_token}"}
+        self.headers = {"Authorization": f"{self.api_key}"}
 
-        tools: List[Any] = []
-        if get_user_details:
-            tools.append(self.get_user_details)
-        if get_teams_details:
-            tools.append(self.get_teams_details)
-        if get_issue_details:
-            tools.append(self.get_issue_details)
-        if create_issue:
-            tools.append(self.create_issue)
-        if update_issue:
-            tools.append(self.update_issue)
-        if get_user_assigned_issues:
-            tools.append(self.get_user_assigned_issues)
-        if get_workflow_issues:
-            tools.append(self.get_workflow_issues)
-        if get_high_priority_issues:
-            tools.append(self.get_high_priority_issues)
+        tools: List[Any] = [
+            self.get_user_details,
+            self.get_teams_details,
+            self.get_issue_details,
+            self.create_issue,
+            self.update_issue,
+            self.get_user_assigned_issues,
+            self.get_workflow_issues,
+            self.get_high_priority_issues,
+        ]
 
-        super().__init__(name="linear tools", tools=tools, **kwargs)
+        super().__init__(name="linear_tools", tools=tools, **kwargs)
 
     def _execute_query(self, query, variables=None):
         """Helper method to execute GraphQL queries with optional variables."""
@@ -217,7 +202,7 @@ class LinearTools(Toolkit):
         """
 
         query = """
-        mutation IssueCreate ($title: String!, $description: String!, $teamId: String!, $projectId: String!, $assigneeId: String!){
+        mutation IssueCreate ($title: String!, $description: String!, $teamId: String!, $projectId: String, $assigneeId: String){
           issueCreate(
             input: { title: $title, description: $description, teamId: $teamId, projectId: $projectId, assigneeId: $assigneeId}
           ) {
