@@ -14,9 +14,14 @@ class NewspaperTools(Toolkit):
         get_article_text (bool): Whether to get the text of an article from a URL.
     """
 
-    def __init__(self, get_article_text: bool = True, **kwargs):
+    def __init__(
+        self,
+        enable_get_article_text: bool = True,
+        all: bool = False,
+        **kwargs,
+    ):
         tools = []
-        if get_article_text:
+        if all or enable_get_article_text:
             tools.append(self.get_article_text)
 
         super().__init__(name="newspaper_toolkit", tools=tools, **kwargs)
